import os
import dotenv
from dotenv import load_dotenv, find_dotenv, dotenv_values

from .config import install_dir


class EnvParser(object):

    def __init__(self, env_file=None):
        if env_file is not None:
            self.env_file = env_file
        else:
            self.env_file = self.get_env_file()
        self.load_env()

    def load_env(self):
        load_dotenv(self.env_file)

    """
    Find it under the root directory path/to/.env
    """
    def get_env_file(self):
        env_file = find_dotenv()
        if not os.path.exists(env_file):
            # /opt/shinetech/stdocker/.env
            env_file = install_dir + '/.env'
        return env_file

    """
    Retrieve all variables to OrderedDict
    """
    def get_values(self):
        return dotenv_values(self.env_file)

    """
    Get value of single variable
    """
    def get_value(self, key):
        return os.getenv(key)

    """
    Create or modify variables
    NOTE: need to remove quotes
        Correct: DOCUMENT_ROOT=/home/sunfeng/stdocker/www
        Incorrect: DOCUMENT_ROOT="/home/sunfeng/stdocker/www"
    In auto mode, don't add quotes if the value is only made of alphanumeric characters
    https://github.com/theskumar/python-dotenv/compare/v0.17.1...v0.18.0
    """
    def set_value(self, key, value):
        return dotenv.set_key(self.env_file, key, value, quote_mode='auto')





import os

# Shinetech Docker installation directory
install_dir = '/opt/shinetech/stdocker'

# The directory where the stdocker command is currently executed
current_dir = os.getcwd()

base_domain = '.dev.php9.cc'

platforms = ['generic', 'magento', 'zend', 'symfony', 'laravel', 'yii', 'slim', 'wordpress']



import os
import sys
import re
from typing import Any, Dict, List
from .utils import list_env_names, get_default_workspace, convert_version
from .env_handler import EnvHandler

try:
    import click
except ImportError:
    sys.stderr.write('It seems stdocker is not installed with cli option. \n'
                     'Run pip3 install "stdocker[cli]" to fix this.')
    sys.exit(1)

from .version import __version__
from .config import install_dir
from .config import current_dir
from .config import base_domain
from .config import platforms


"""
Check if project name is valid
"""
def check_project_name(ctx, param, value):
    if not param.required and value is None:
        return value
    pattern = re.compile('^[a-zA-Z][0-9a-zA-Z]{0,50}$')
    matched = pattern.match(value)
    if matched is None:
        click.echo(
            click.style(f"Invalid project name: Limit length to 1-50 characters (1 character in front + up to 49 characters in back)",
                        fg='red'))
        ctx.abort()
    return value


@click.group()
@click.option('-d', '--working-dir', default=install_dir,
              type=click.Path(dir_okay=True),
              help="Location of the installation directory, defaults to " + install_dir + ".")
@click.version_option(version=__version__)
@click.pass_context
def cli(ctx: click.Context, working_dir: Any) -> None:
    """Shinetech Docker CLI"""
    ctx.obj = {}
    ctx.obj['WORKING_DIR'] = working_dir
    ctx.obj['CURRENT_DIR'] = current_dir
    # All scripts will based on working dir to run
    os.chdir(working_dir)  # Debug directory: /var/www/html/Shinetech/shinetech-docker


@cli.command()
@click.pass_context
@click.argument('command', required=True)
def run(ctx: click.Context, command: Any) -> None:
    """Execute sudo docker * command"""
    os.system('sudo docker ' + command)


@cli.command()
@click.pass_context
@click.argument('command', required=True)
def exec(ctx: click.Context, command: Any) -> None:
    """Execute sudo docker exec * command"""
    os.system('sudo docker exec ' + command)


@cli.command()
@click.pass_context
@click.argument('command', required=True)
def compose(ctx: click.Context, command: Any) -> None:
    """Execute sudo docker-compose * command"""
    os.system('sudo docker-compose ' + command)


@cli.command()
@click.pass_context
@click.argument('service', required=True)
def restart(ctx: click.Context, service: Any) -> None:
    """Restart specified docker service"""
    os.system('sudo docker-compose restart ' + service)


@cli.command()
@click.pass_context
def configure(ctx: click.Context) -> None:
    """Configure local environment, will guide you through creating your .env and docker-compose.yml."""
    os.system('python3 install/configurator.py')


@cli.command()
@click.pass_context
@click.option('--env', default='magento_244',
              type=click.Choice(list_env_names(install_dir)),
              help="Build the development environment based on the specified configuration.")
def build(ctx: click.Context, env: Any) -> None:
    """Build local development environment with your configuration"""
    os.system('bash builder.sh ' + env)


@cli.command()
@click.pass_context
def start(ctx: click.Context) -> None:
    """Launch docker services"""
    click.echo(click.style(f"Start to launch docker services", fg='green'))
    os.system('bash bin/launch.sh')


@cli.command()
@click.pass_context
def stop(ctx: click.Context) -> None:
    """Stop docker services"""
    click.echo(click.style(f"Start to stop docker services", fg='green'))
    os.system('bash bin/stop.sh')


@cli.command()
@click.pass_context
@click.option('--dbname', required=True,
              help="Specify the database name to export or import.")
@click.option('--backup-sql-file',
              help="Specifies the backup SQL file path, "
                   "which can be used for imported source file and exported target file or directory. "
                   "The default is the specified file in the current directory.")
@click.argument('action', type=click.Choice(['import', 'export']), required=True)
def database(ctx: click.Context, action: Any, dbname: Any, backup_sql_file: Any) -> None:
    """Export or import database"""
    if dbname is None:
        click.echo(click.style(f"Invalid database name", fg='red'))
        exit(1)
    if action == 'import':
        if backup_sql_file is None:
            click.echo(click.style(f"Invalid source backup SQL file", fg='red'))
            exit(1)
        os.system('bash bin/import_db.sh ' + dbname + ' ' + backup_sql_file + ' ' + current_dir)
    elif action == 'export':
        command = 'bash bin/export_db.sh ' + dbname + ' ' + current_dir
        if backup_sql_file is not None:
            command += ' ' + backup_sql_file
        os.system(command)


@cli.command()
@click.pass_context
@click.argument('service', required=True)
def bash(ctx: click.Context, service: Any) -> None:
    """Bash session for running container"""
    os.system('bash bin/container_bash.sh ' + service)


@cli.command()
@click.pass_context
@click.option('--target-version',
              help="Specify the target version to upgrade. e.g: 1.0.1")
@click.option('--force/--no-force', default=False,
              help="Force or not force restart of services")
def upgrade(ctx: click.Context, target_version: Any, force: Any) -> None:
    """Upgrade Shinetech Docker"""
    command = 'bash bin/upgrade.sh '

    if force:
        command += ' y'
    else:
        command += ' n'

    if target_version is not None:
        command += ' ' + target_version

    os.system(command)

    click.echo(click.style(f"Upgraded successfully.", fg='green'))


@cli.command()
@click.pass_context
@click.option('--directory', default=get_default_workspace(),
              help="Specify the workspace directory. default is " + get_default_workspace())
def workspace(ctx: click.Context, directory: Any) -> None:
    """Initial workspace"""
    if directory is not None:
        os.system('bash bin/workspace.sh ' + directory)
    else:
        os.system('bash bin/workspace.sh')


@cli.command()
@click.pass_context
def about(ctx: click.Context) -> None:
    """Show the local environment and workspace information"""
    working_dir = ctx.obj['WORKING_DIR']
    env_handler = EnvHandler(working_dir=working_dir)
    env_values = env_handler.get_env_values()

    click.echo(click.style(f"Current environment:", fg='yellow', bold=True))
    click.echo(click.style(f" - {env_values['DEFAULT_ENV']}", fg='cyan'))

    click.echo(click.style(f"Your workspace information:", fg='yellow', bold=True))
    click.echo(click.style(f" - Workspace: {env_values['WORKSPACE']}", fg='cyan'))
    click.echo(
        click.style(f" - Project Directory: {env_values['DOCUMENT_ROOT']}", fg='cyan'))
    click.echo(click.style(f" - Nginx VHosts: {env_values['NGINX_VHOSTS_DIR']}", fg='cyan'))
    click.echo(click.style(f" - Nginx Log: {env_values['NGINX_LOG_DIR']}", fg='cyan'))
    click.echo(
        click.style(f" - Apache VHosts: {env_values['APACHE_VHOSTS_DIR']}", fg='cyan'))
    click.echo(click.style(f" - Apache Log: {env_values['APACHE_LOG_DIR']}", fg='cyan'))
    click.echo(click.style(f" - SSL CA: {env_values['SSL_CA_DIR']}", fg='cyan'))
    click.echo(click.style(f" - php.ini: {env_values['PHP_INI']}", fg='cyan'))
    click.echo(click.style(f" - MySQL Log: {env_values['MYSQL_LOG_DIR']}", fg='cyan'))


@cli.command()
@click.pass_context
def envs(ctx: click.Context) -> None:
    """List all environments"""
    working_dir = ctx.obj['WORKING_DIR']
    env_handler = EnvHandler(working_dir=working_dir)
    env_table = env_handler.list_env_table()
    print(env_table)


@cli.command()
@click.pass_context
@click.option('--platform', required=True, default='generic',
              type=click.Choice(platforms),
              help="Specifies the framework used by the project.")
@click.option('--name', required=True,
              callback=check_project_name,
              help="Specify project name.")
@click.option('--db-sql-file', required=False,
              help="SQL backup file for initializing the database.")
@click.option('--country', required=False,
              help="Build project by country, such as HP project have multiple independent countries and regions.")
@click.option('--multiple-domain/--no-multiple-domain', default=False,
              help="Use multiple domains for multiple sites, single domain is used by default. "
                   "This option takes effect if --country is not empty. "
                   "e.g: For HP project, If No will use hp.dev.php9.cc for all country sites, "
                   "if Yes will use <country>.hp.dev.php9.cc for different sites.")
def init_project(ctx: click.Context, platform: Any, name: Any, db_sql_file: Any, country: Any, multiple_domain: Any) -> None:
    """Create a project based on a base template or existing code"""
    working_dir = ctx.obj['WORKING_DIR']
    env_handler = EnvHandler(working_dir=working_dir)
    env_values = env_handler.get_env_values()
    workspace_dir = env_values['WORKSPACE']
    current_env_configs = env_handler.get_current_env_configs()
    webserver = current_env_configs['services']['webserver']

    domain = name + base_domain
    if multiple_domain and country is not None:
        domain = name + '-' + country + base_domain

    project_dir = workspace_dir + '/www/' + domain
    if os.path.exists(project_dir):
        click.echo(click.style(f"NOTE: The project directory {project_dir} already exists.", fg='cyan'))
        click.confirm('Do you confirm to override and create project?', abort=True)

    command = 'bash bin/init_project.sh ' \
              + current_dir + ' ' + workspace_dir + ' ' + webserver + ' ' + platform + ' ' + name

    if db_sql_file is not None:
        command += ' ' + db_sql_file
    else:
        command += ' ""'

    if country is not None:
        command += ' ' + country
        if multiple_domain:
            command += ' y'
        else:
            command += ' n'

    os.system(command)


@cli.command()
@click.pass_context
@click.option('--target-version', required=True,
              help="Specify Magento version. e.g: 2.4.5, 2.4.4-p1")
@click.option('--source-code-file', required=False,
              help="Specify Magento original source code file. "
                   "If not specified, composer will be used for installation, "
                   "otherwise the specified source code package will be used.")
@click.option('--project-name', required=False,
              callback=check_project_name,
              help="Specify project name.")
def init_magento(ctx: click.Context, target_version: Any, source_code_file: Any, project_name: Any) -> None:
    """Create a project based on the Magento source code"""
    working_dir = ctx.obj['WORKING_DIR']
    env_handler = EnvHandler(working_dir=working_dir)
    env_values = env_handler.get_env_values()
    workspace_dir = env_values['WORKSPACE']
    current_env_configs = env_handler.get_current_env_configs()
    webserver = current_env_configs['services']['webserver']

    if target_version is not None:
        # 2.4.5 > 245
        version = convert_version(target_version)
        project_name = 'm' + version  # e.g: m245

    project_domain = project_name + base_domain

    project_dir = workspace_dir + '/www/' + project_domain
    if os.path.exists(project_dir):
        click.echo(click.style(f"NOTE: The Magento project {project_dir} already exists.", fg='cyan'))
        click.confirm('Do you confirm to override and create project?', abort=True)

    command = 'bash bin/init_magento.sh ' \
              + current_dir + ' ' + workspace_dir + ' ' + webserver + ' ' + project_name + ' ' + target_version
    if source_code_file is not None:
        command += ' ' + source_code_file

    os.system(command)


@cli.command()
@click.pass_context
def status(ctx: click.Context) -> None:
    """List all running containers"""
    os.system('sudo docker-compose ps')


def main():
    cli()


if __name__ == '__main__':
    main()



from pathlib import Path
from .env_handler import EnvHandler


"""
List all environment names
"""
def list_env_names(install_dir):
    env_handler = EnvHandler(working_dir=install_dir)
    return env_handler.list_customized_env_names()


"""
Get current user home directory
"""
def get_default_workspace():
    return str(Path.home()) + '/stdocker'


"""
Convert version
e.g: 
2.4.5 > 245
2.4.3-p2 > 243p2
"""
def convert_version(version):
    version = version.replace(".", "").replace("-", "").replace("_", "")
    return version



import yaml


class YamlParser(object):

    def __init__(self, yaml_file):
        self.yaml_file = yaml_file

    def load_file(self):
        data = {}

        with open(self.yaml_file, "r") as stream:
            try:
                data = yaml.safe_load(stream)
            except yaml.YAMLError as exc:
                print(exc)

        return data

    def save_dict_to_yaml(self, dict_data: dict):
        with open(self.yaml_file, 'w') as file:
            file.write(yaml.dump(dict_data, allow_unicode=True, sort_keys=False))
        return self.yaml_file

    def load_yaml_to_dict(self):
        with open(self.yaml_file) as file:
            dict_data = yaml.load(file.read(), Loader=yaml.FullLoader)
            return dict_data




from setuptools import setup, find_packages

def read_files(files):
    data = []
    for file in files:
        with open(file, encoding='utf-8') as f:
            data.append(f.read())
    return "\n".join(data)

long_description = read_files(['README.md', 'CHANGELOG.md'])

requirements = []
with open('requirements.txt') as f:
    requirements = f.readlines()

meta = {}
with open('./stdocker/version.py', encoding='utf-8') as f:
    exec(f.read(), meta)

setup(
    name='stdocker',
    version=meta['__version__'],
    author='Jason Sun',
    author_email='sunf@shinetechsoftware.com',
    url='https://github.com/winds1983/stdocker',
    description='Shinetech Docker CLI.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='MIT',
    packages=find_packages(),
    python_requires=">=3.8",
    extras_require={
        'cli': ['click>=5.0', ],
    },
    entry_points={
        'console_scripts': [
            'stdocker=stdocker.cli:main'
        ]
    },
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: Implementation :: PyPy',
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords=['python package', 'stdocker', 'shinetech', 'docker', 'development environment'],
    install_requires=requirements,
    zip_safe=False
)



import os
from beautifultable import BeautifulTable
from termcolor import colored
from .yaml_parser import YamlParser
from .env_parser import EnvParser

from .config import install_dir


class EnvHandler(object):

    def __init__(self, working_dir=None):
        if working_dir is None:
            self.working_dir = install_dir
        else:
            self.working_dir = working_dir

    """
    Get all env configs
    """
    def get_env_values(self):
        env_file = self.working_dir + '/.env'
        env_handler = EnvParser(env_file=env_file)
        return env_handler.get_values()

    """
    Get current env from .env
    """
    def get_current_env(self):
        env_values = self.get_env_values()
        return env_values['DEFAULT_ENV']

    """
    Get configs for current env
    Will convert yaml to dict
    """
    def get_current_env_configs(self):
        config_file = self.get_current_env_config_file()
        yaml_parser = YamlParser(yaml_file=config_file)
        return yaml_parser.load_yaml_to_dict()

    """
    Get config file for current env
    e.g: /opt/shinetech/stdocker/config/env/templates/magento_244.yml
    """
    def get_current_env_config_file(self):
        current_env = self.get_current_env()

        custom_env_dir = self.working_dir + '/var/env'
        predefined_env_dir = self.working_dir + '/config/env/templates'

        config_file = predefined_env_dir + '/' + current_env + '.yml'
        if not os.path.exists(config_file):
            config_file = custom_env_dir + '/' + current_env + '.yml'
            if not os.path.exists(config_file):
                return None

        return config_file

    """
    List all customized environment names
    """
    def list_customized_env_names(self):
        env_files = self.list_customized_env_files()

        envs = []
        for env_file in env_files:
            if env_file != '.gitkeep':
                env = env_file.split('.')
                envs.append(env[0])

        envs.sort(reverse=False)

        return envs

    """
    List detailed config for all customized environments
    """
    def list_customized_detailed_envs(self):
        env_files = self.list_customized_env_files(with_dir=True)

        envs = []
        for env_file in env_files:
            if '.gitkeep' not in env_file:
                yaml_parser = YamlParser(yaml_file=env_file)
                detail_data = yaml_parser.load_yaml_to_dict()

                filename = os.path.basename(env_file)
                env_name = filename.split('.')
                env = env_name[0]
                detail_data['env'] = env

                envs.append(detail_data)

        return envs

    """
    List files for all customized environments
    """
    def list_customized_env_files(self, with_dir=False):
        custom_env_dir = self.working_dir + '/var/env'
        predefined_env_dir = self.working_dir + '/config/env/templates'

        custom_env_files = os.listdir(custom_env_dir)
        predefined_env_files = os.listdir(predefined_env_dir)

        env_files = []
        if with_dir:
            for filename in predefined_env_files:
                file_path = predefined_env_dir + '/' + filename
                env_files.append(file_path)

            for filename in custom_env_files:
                file_path = custom_env_dir + '/' + filename
                env_files.append(file_path)
        else:
            env_files = predefined_env_files + custom_env_files

        env_files.sort(reverse=False)

        return env_files

    def list_env_table(self):
        current_env = self.get_current_env()
        customized_envs = self.list_customized_detailed_envs()
        # env_names = self.list_customized_env_names()
        services = ['webserver', 'php', 'mysql', 'mariadb',
                    'phpmyadmin', 'elasticsearch', 'elasticvue',
                    'redis', 'phpredisadmin', 'rabbitmq', 'mailcatcher',
                    'mongo', 'mongoexpress', 'postgres', 'pgweb', 'memcached']
        service_names = ['Web Server', 'PHP', 'MySQL', 'MariaDB',
                         'phpMyAdmin', 'Elasticsearch', 'Elasticvue',
                         'Redis', 'phpRedisAdmin', 'RabbitMQ', 'Mailcatcher',
                         'MongoDB', 'Mongo Express', 'PostgreSQL', 'Pgweb', 'Memcached']

        table = BeautifulTable(maxwidth=200)
        columns_headers = ['Env Code'] + service_names

        env_detail_names = []
        for item in customized_envs:
            env_detail_names.append(item['name'])
            env = item['env']
            # description = item['description']

            services_status = []
            if env == current_env:
                services_status.append(colored(env, 'red'))
            else:
                services_status.append(env)

            for service in services:
                status = 'N'
                if service in item['services'].keys():
                    if item['services'][service] is True:
                        status = 'Y'
                    else:
                        status = item['services'][service]

                if env == current_env:
                    services_status.append(colored(status, 'red'))
                else:
                    services_status.append(status)

            table.rows.append(services_status)

        table.rows.header = env_detail_names
        table.columns.header = columns_headers

        return table
